@@ -152,7 +152,7 @@ function getFullURL($loc) {
 	// make sure we have a directory to go to
 	if (empty($loc)) {
 		$loc = '/';
-	} else if ($loc{0} != '/') {
+	} else if ($loc[0] != '/') {
 		$loc = '/'.$loc;
 	}
 
@@ -222,7 +222,7 @@ function expandTabs($s, $tabwidth = - 1) {
 	for ($i = 0; $i < $chunkscount; $i++) {
 		// make sure we're not dealing with an empty string
 		if (empty($chunks[$i])) continue;
-		switch ($chunks[$i]{0}) {
+		switch ($chunks[$i][0]) {
 			case '<': // HTML tag: ignore its width by doing nothing
 				break;
 
@@ -399,6 +399,8 @@ function getUserLanguage($languages, $default, $userchoice) {
 
 // }}}
 
+// {{{ tempnameWithCheck
+
 function tempnamWithCheck($dir, $prefix) {
 	$tmp = tempnam($dir, $prefix);
 
@@ -416,3 +418,5 @@ function tempnamWithCheck($dir, $prefix) {
 
 	return $tmp;
 }
+
+// }}}
\ No newline at end of file
