@@ -116,3 +116,7 @@ $lang["COMPPATH"] = "Compare Path:";
 $lang["WITHPATH"] = "With Path:";
 $lang["FILEDELETED"] = "File deleted";
 $lang["FILEADDED"] = "New file";
+
+$lang['SEARCH'] = "Search";
+$lang['SEARCH_PLACEHOLDER'] = "Search for items here";
+$lang['NOITEMSFOUND'] = "No Items Found";
\ No newline at end of file
